# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'getxpubdialog.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_GetXpubDialog(object):
    def setupUi(self, GetXpubDialog):
        if not GetXpubDialog.objectName():
            GetXpubDialog.setObjectName(u"GetXpubDialog")
        GetXpubDialog.resize(1050, 125)
        self.gridLayout = QGridLayout(GetXpubDialog)
        self.gridLayout.setSpacing(10)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(10, 10, 10, 10)
        self.buttonBox = QDialogButtonBox(GetXpubDialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setMaximumSize(QSize(200, 16777215))
        self.buttonBox.setFocusPolicy(Qt.NoFocus)
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Close)

        self.gridLayout.addWidget(self.buttonBox, 2, 2, 1, 1)

        self.path_label = QLabel(GetXpubDialog)
        self.path_label.setObjectName(u"path_label")

        self.gridLayout.addWidget(self.path_label, 0, 0, 1, 1)

        self.xpub_textedit = QTextEdit(GetXpubDialog)
        self.xpub_textedit.setObjectName(u"xpub_textedit")
        self.xpub_textedit.setMinimumSize(QSize(0, 30))
        self.xpub_textedit.setMaximumSize(QSize(16777215, 30))
        self.xpub_textedit.setLineWrapMode(QTextEdit.NoWrap)
        self.xpub_textedit.setReadOnly(True)

        self.gridLayout.addWidget(self.xpub_textedit, 1, 1, 1, 2)

        self.path_lineedit = QLineEdit(GetXpubDialog)
        self.path_lineedit.setObjectName(u"path_lineedit")
        self.path_lineedit.setMinimumSize(QSize(200, 30))

        self.gridLayout.addWidget(self.path_lineedit, 0, 1, 1, 1)

        self.getxpub_button = QPushButton(GetXpubDialog)
        self.getxpub_button.setObjectName(u"getxpub_button")
        self.getxpub_button.setMaximumSize(QSize(200, 16777215))

        self.gridLayout.addWidget(self.getxpub_button, 0, 2, 1, 1)

        self.xpub_label = QLabel(GetXpubDialog)
        self.xpub_label.setObjectName(u"xpub_label")
        self.xpub_label.setAlignment(Qt.AlignCenter)

        self.gridLayout.addWidget(self.xpub_label, 1, 0, 1, 1)


        self.retranslateUi(GetXpubDialog)

        QMetaObject.connectSlotsByName(GetXpubDialog)
    # setupUi

    def retranslateUi(self, GetXpubDialog):
        GetXpubDialog.setWindowTitle(QCoreApplication.translate("GetXpubDialog", u"Dialog", None))
        self.path_label.setText(QCoreApplication.translate("GetXpubDialog", u"Derivation Path", None))
        self.getxpub_button.setText(QCoreApplication.translate("GetXpubDialog", u"Get xpub", None))
        self.xpub_label.setText(QCoreApplication.translate("GetXpubDialog", u"xpub", None))
    # retranslateUi




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: perform_attestation.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19perform_attestation.proto\x12\x14shiftcrypto.bitbox02\".\n\x19PerformAttestationRequest\x12\x11\n\tchallenge\x18\x01 \x01(\x0c\"\x9e\x01\n\x1aPerformAttestationResponse\x12\x17\n\x0f\x62ootloader_hash\x18\x01 \x01(\x0c\x12\x15\n\rdevice_pubkey\x18\x02 \x01(\x0c\x12\x13\n\x0b\x63\x65rtificate\x18\x03 \x01(\x0c\x12\x1e\n\x16root_pubkey_identifier\x18\x04 \x01(\x0c\x12\x1b\n\x13\x63hallenge_signature\x18\x05 \x01(\x0c\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'perform_attestation_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PERFORMATTESTATIONREQUEST._serialized_start=51
  _PERFORMATTESTATIONREQUEST._serialized_end=97
  _PERFORMATTESTATIONRESPONSE._serialized_start=100
  _PERFORMATTESTATIONRESPONSE._serialized_end=258
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
from . import antiklepto_pb2
import builtins
from . import common_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _BTCCoin:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _BTCCoinEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_BTCCoin.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    BTC: _BTCCoin.ValueType  # 0
    TBTC: _BTCCoin.ValueType  # 1
    LTC: _BTCCoin.ValueType  # 2
    TLTC: _BTCCoin.ValueType  # 3
class BTCCoin(_BTCCoin, metaclass=_BTCCoinEnumTypeWrapper):
    pass

BTC: BTCCoin.ValueType  # 0
TBTC: BTCCoin.ValueType  # 1
LTC: BTCCoin.ValueType  # 2
TLTC: BTCCoin.ValueType  # 3
global___BTCCoin = BTCCoin


class _BTCOutputType:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _BTCOutputTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_BTCOutputType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _BTCOutputType.ValueType  # 0
    P2PKH: _BTCOutputType.ValueType  # 1
    P2SH: _BTCOutputType.ValueType  # 2
    P2WPKH: _BTCOutputType.ValueType  # 3
    P2WSH: _BTCOutputType.ValueType  # 4
    P2TR: _BTCOutputType.ValueType  # 5
class BTCOutputType(_BTCOutputType, metaclass=_BTCOutputTypeEnumTypeWrapper):
    pass

UNKNOWN: BTCOutputType.ValueType  # 0
P2PKH: BTCOutputType.ValueType  # 1
P2SH: BTCOutputType.ValueType  # 2
P2WPKH: BTCOutputType.ValueType  # 3
P2WSH: BTCOutputType.ValueType  # 4
P2TR: BTCOutputType.ValueType  # 5
global___BTCOutputType = BTCOutputType


class BTCScriptConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _SimpleType:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _SimpleTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCScriptConfig._SimpleType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        P2WPKH_P2SH: BTCScriptConfig._SimpleType.ValueType  # 0
        P2WPKH: BTCScriptConfig._SimpleType.ValueType  # 1
        P2TR: BTCScriptConfig._SimpleType.ValueType  # 2
    class SimpleType(_SimpleType, metaclass=_SimpleTypeEnumTypeWrapper):
        """SimpleType is a "simple" script: one public key, no additional inputs."""
        pass

    P2WPKH_P2SH: BTCScriptConfig.SimpleType.ValueType  # 0
    P2WPKH: BTCScriptConfig.SimpleType.ValueType  # 1
    P2TR: BTCScriptConfig.SimpleType.ValueType  # 2

    class Multisig(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        class _ScriptType:
            ValueType = typing.NewType('ValueType', builtins.int)
            V: typing_extensions.TypeAlias = ValueType
        class _ScriptTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCScriptConfig.Multisig._ScriptType.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            P2WSH: BTCScriptConfig.Multisig._ScriptType.ValueType  # 0
            """native segwit v0 multisig (bech32 addresses)"""

            P2WSH_P2SH: BTCScriptConfig.Multisig._ScriptType.ValueType  # 1
            """wrapped segwit for legacy address compatibility"""

        class ScriptType(_ScriptType, metaclass=_ScriptTypeEnumTypeWrapper):
            pass

        P2WSH: BTCScriptConfig.Multisig.ScriptType.ValueType  # 0
        """native segwit v0 multisig (bech32 addresses)"""

        P2WSH_P2SH: BTCScriptConfig.Multisig.ScriptType.ValueType  # 1
        """wrapped segwit for legacy address compatibility"""


        THRESHOLD_FIELD_NUMBER: builtins.int
        XPUBS_FIELD_NUMBER: builtins.int
        OUR_XPUB_INDEX_FIELD_NUMBER: builtins.int
        SCRIPT_TYPE_FIELD_NUMBER: builtins.int
        threshold: builtins.int
        @property
        def xpubs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common_pb2.XPub]:
            """xpubs are acount-level xpubs. Addresses are going to be derived from it using: m/<change>/<receive>.
            The number of xpubs defines the number of cosigners.
            """
            pass
        our_xpub_index: builtins.int
        """Index to the xpub of our keystore in xpubs. The keypath to it is provided via
        BTCPubRequest/BTCSignInit.
        """

        script_type: global___BTCScriptConfig.Multisig.ScriptType.ValueType
        def __init__(self,
            *,
            threshold: builtins.int = ...,
            xpubs: typing.Optional[typing.Iterable[common_pb2.XPub]] = ...,
            our_xpub_index: builtins.int = ...,
            script_type: global___BTCScriptConfig.Multisig.ScriptType.ValueType = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["our_xpub_index",b"our_xpub_index","script_type",b"script_type","threshold",b"threshold","xpubs",b"xpubs"]) -> None: ...

    SIMPLE_TYPE_FIELD_NUMBER: builtins.int
    MULTISIG_FIELD_NUMBER: builtins.int
    simple_type: global___BTCScriptConfig.SimpleType.ValueType
    @property
    def multisig(self) -> global___BTCScriptConfig.Multisig: ...
    def __init__(self,
        *,
        simple_type: global___BTCScriptConfig.SimpleType.ValueType = ...,
        multisig: typing.Optional[global___BTCScriptConfig.Multisig] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["config",b"config","multisig",b"multisig","simple_type",b"simple_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["config",b"config","multisig",b"multisig","simple_type",b"simple_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["config",b"config"]) -> typing.Optional[typing_extensions.Literal["simple_type","multisig"]]: ...
global___BTCScriptConfig = BTCScriptConfig

class BTCPubRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _XPubType:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _XPubTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCPubRequest._XPubType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        TPUB: BTCPubRequest._XPubType.ValueType  # 0
        XPUB: BTCPubRequest._XPubType.ValueType  # 1
        YPUB: BTCPubRequest._XPubType.ValueType  # 2
        ZPUB: BTCPubRequest._XPubType.ValueType  # 3
        """zpub"""

        VPUB: BTCPubRequest._XPubType.ValueType  # 4
        """vpub"""

        UPUB: BTCPubRequest._XPubType.ValueType  # 5
        CAPITAL_VPUB: BTCPubRequest._XPubType.ValueType  # 6
        """Vpub"""

        CAPITAL_ZPUB: BTCPubRequest._XPubType.ValueType  # 7
        """Zpub"""

        CAPITAL_UPUB: BTCPubRequest._XPubType.ValueType  # 8
        """Upub"""

        CAPITAL_YPUB: BTCPubRequest._XPubType.ValueType  # 9
        """Ypub"""

    class XPubType(_XPubType, metaclass=_XPubTypeEnumTypeWrapper):
        pass

    TPUB: BTCPubRequest.XPubType.ValueType  # 0
    XPUB: BTCPubRequest.XPubType.ValueType  # 1
    YPUB: BTCPubRequest.XPubType.ValueType  # 2
    ZPUB: BTCPubRequest.XPubType.ValueType  # 3
    """zpub"""

    VPUB: BTCPubRequest.XPubType.ValueType  # 4
    """vpub"""

    UPUB: BTCPubRequest.XPubType.ValueType  # 5
    CAPITAL_VPUB: BTCPubRequest.XPubType.ValueType  # 6
    """Vpub"""

    CAPITAL_ZPUB: BTCPubRequest.XPubType.ValueType  # 7
    """Zpub"""

    CAPITAL_UPUB: BTCPubRequest.XPubType.ValueType  # 8
    """Upub"""

    CAPITAL_YPUB: BTCPubRequest.XPubType.ValueType  # 9
    """Ypub"""


    COIN_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    XPUB_TYPE_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    DISPLAY_FIELD_NUMBER: builtins.int
    coin: global___BTCCoin.ValueType
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    xpub_type: global___BTCPubRequest.XPubType.ValueType
    @property
    def script_config(self) -> global___BTCScriptConfig: ...
    display: builtins.bool
    def __init__(self,
        *,
        coin: global___BTCCoin.ValueType = ...,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        xpub_type: global___BTCPubRequest.XPubType.ValueType = ...,
        script_config: typing.Optional[global___BTCScriptConfig] = ...,
        display: builtins.bool = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["output",b"output","script_config",b"script_config","xpub_type",b"xpub_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coin",b"coin","display",b"display","keypath",b"keypath","output",b"output","script_config",b"script_config","xpub_type",b"xpub_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["output",b"output"]) -> typing.Optional[typing_extensions.Literal["xpub_type","script_config"]]: ...
global___BTCPubRequest = BTCPubRequest

class BTCScriptConfigWithKeypath(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    @property
    def script_config(self) -> global___BTCScriptConfig: ...
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(self,
        *,
        script_config: typing.Optional[global___BTCScriptConfig] = ...,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["script_config",b"script_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["keypath",b"keypath","script_config",b"script_config"]) -> None: ...
global___BTCScriptConfigWithKeypath = BTCScriptConfigWithKeypath

class BTCSignInitRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _FormatUnit:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _FormatUnitEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCSignInitRequest._FormatUnit.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT: BTCSignInitRequest._FormatUnit.ValueType  # 0
        """According to `coin` (BTC, LTC, etc.)."""

        SAT: BTCSignInitRequest._FormatUnit.ValueType  # 1
        """Only valid for BTC/TBTC, formats as "sat"/"tsat"."""

    class FormatUnit(_FormatUnit, metaclass=_FormatUnitEnumTypeWrapper):
        pass

    DEFAULT: BTCSignInitRequest.FormatUnit.ValueType  # 0
    """According to `coin` (BTC, LTC, etc.)."""

    SAT: BTCSignInitRequest.FormatUnit.ValueType  # 1
    """Only valid for BTC/TBTC, formats as "sat"/"tsat"."""


    COIN_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIGS_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    NUM_INPUTS_FIELD_NUMBER: builtins.int
    NUM_OUTPUTS_FIELD_NUMBER: builtins.int
    LOCKTIME_FIELD_NUMBER: builtins.int
    FORMAT_UNIT_FIELD_NUMBER: builtins.int
    coin: global___BTCCoin.ValueType
    @property
    def script_configs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BTCScriptConfigWithKeypath]:
        """used script configs in inputs and changes"""
        pass
    version: builtins.int
    """must be 1 or 2"""

    num_inputs: builtins.int
    num_outputs: builtins.int
    locktime: builtins.int
    """must be <500000000"""

    format_unit: global___BTCSignInitRequest.FormatUnit.ValueType
    def __init__(self,
        *,
        coin: global___BTCCoin.ValueType = ...,
        script_configs: typing.Optional[typing.Iterable[global___BTCScriptConfigWithKeypath]] = ...,
        version: builtins.int = ...,
        num_inputs: builtins.int = ...,
        num_outputs: builtins.int = ...,
        locktime: builtins.int = ...,
        format_unit: global___BTCSignInitRequest.FormatUnit.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["coin",b"coin","format_unit",b"format_unit","locktime",b"locktime","num_inputs",b"num_inputs","num_outputs",b"num_outputs","script_configs",b"script_configs","version",b"version"]) -> None: ...
global___BTCSignInitRequest = BTCSignInitRequest

class BTCSignNextResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Type:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCSignNextResponse._Type.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        INPUT: BTCSignNextResponse._Type.ValueType  # 0
        OUTPUT: BTCSignNextResponse._Type.ValueType  # 1
        DONE: BTCSignNextResponse._Type.ValueType  # 2
        PREVTX_INIT: BTCSignNextResponse._Type.ValueType  # 3
        """For the previous transaction at input `index`."""

        PREVTX_INPUT: BTCSignNextResponse._Type.ValueType  # 4
        PREVTX_OUTPUT: BTCSignNextResponse._Type.ValueType  # 5
        HOST_NONCE: BTCSignNextResponse._Type.ValueType  # 6
    class Type(_Type, metaclass=_TypeEnumTypeWrapper):
        pass

    INPUT: BTCSignNextResponse.Type.ValueType  # 0
    OUTPUT: BTCSignNextResponse.Type.ValueType  # 1
    DONE: BTCSignNextResponse.Type.ValueType  # 2
    PREVTX_INIT: BTCSignNextResponse.Type.ValueType  # 3
    """For the previous transaction at input `index`."""

    PREVTX_INPUT: BTCSignNextResponse.Type.ValueType  # 4
    PREVTX_OUTPUT: BTCSignNextResponse.Type.ValueType  # 5
    HOST_NONCE: BTCSignNextResponse.Type.ValueType  # 6

    TYPE_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    HAS_SIGNATURE_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    PREV_INDEX_FIELD_NUMBER: builtins.int
    ANTI_KLEPTO_SIGNER_COMMITMENT_FIELD_NUMBER: builtins.int
    type: global___BTCSignNextResponse.Type.ValueType
    index: builtins.int
    """index of the current input or output"""

    has_signature: builtins.bool
    """only as a response to BTCSignInputRequest"""

    signature: builtins.bytes
    """64 bytes (32 bytes big endian R, 32 bytes big endian S). Only if has_signature is true."""

    prev_index: builtins.int
    """Previous tx's input/output index in case of PREV_INPUT or PREV_OUTPUT, for the input at `index`."""

    @property
    def anti_klepto_signer_commitment(self) -> antiklepto_pb2.AntiKleptoSignerCommitment: ...
    def __init__(self,
        *,
        type: global___BTCSignNextResponse.Type.ValueType = ...,
        index: builtins.int = ...,
        has_signature: builtins.bool = ...,
        signature: builtins.bytes = ...,
        prev_index: builtins.int = ...,
        anti_klepto_signer_commitment: typing.Optional[antiklepto_pb2.AntiKleptoSignerCommitment] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["anti_klepto_signer_commitment",b"anti_klepto_signer_commitment"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["anti_klepto_signer_commitment",b"anti_klepto_signer_commitment","has_signature",b"has_signature","index",b"index","prev_index",b"prev_index","signature",b"signature","type",b"type"]) -> None: ...
global___BTCSignNextResponse = BTCSignNextResponse

class BTCSignInputRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREVOUTHASH_FIELD_NUMBER: builtins.int
    PREVOUTINDEX_FIELD_NUMBER: builtins.int
    PREVOUTVALUE_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_INDEX_FIELD_NUMBER: builtins.int
    HOST_NONCE_COMMITMENT_FIELD_NUMBER: builtins.int
    prevOutHash: builtins.bytes
    prevOutIndex: builtins.int
    prevOutValue: builtins.int
    sequence: builtins.int
    """must be 0xffffffff-2, 0xffffffff-1 or 0xffffffff"""

    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """all inputs must be ours."""
        pass
    script_config_index: builtins.int
    """References a script config from BTCSignInitRequest"""

    @property
    def host_nonce_commitment(self) -> antiklepto_pb2.AntiKleptoHostNonceCommitment: ...
    def __init__(self,
        *,
        prevOutHash: builtins.bytes = ...,
        prevOutIndex: builtins.int = ...,
        prevOutValue: builtins.int = ...,
        sequence: builtins.int = ...,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        script_config_index: builtins.int = ...,
        host_nonce_commitment: typing.Optional[antiklepto_pb2.AntiKleptoHostNonceCommitment] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["host_nonce_commitment",b"host_nonce_commitment"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["host_nonce_commitment",b"host_nonce_commitment","keypath",b"keypath","prevOutHash",b"prevOutHash","prevOutIndex",b"prevOutIndex","prevOutValue",b"prevOutValue","script_config_index",b"script_config_index","sequence",b"sequence"]) -> None: ...
global___BTCSignInputRequest = BTCSignInputRequest

class BTCSignOutputRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OURS_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_INDEX_FIELD_NUMBER: builtins.int
    ours: builtins.bool
    type: global___BTCOutputType.ValueType
    """if ours is false"""

    value: builtins.int
    """20 bytes for p2pkh, p2sh, pw2wpkh. 32 bytes for p2wsh."""

    payload: builtins.bytes
    """if ours is false. Renamed from `hash`."""

    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """if ours is true"""
        pass
    script_config_index: builtins.int
    """If ours is true. References a script config from BTCSignInitRequest"""

    def __init__(self,
        *,
        ours: builtins.bool = ...,
        type: global___BTCOutputType.ValueType = ...,
        value: builtins.int = ...,
        payload: builtins.bytes = ...,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        script_config_index: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["keypath",b"keypath","ours",b"ours","payload",b"payload","script_config_index",b"script_config_index","type",b"type","value",b"value"]) -> None: ...
global___BTCSignOutputRequest = BTCSignOutputRequest

class BTCScriptConfigRegistration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COIN_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    coin: global___BTCCoin.ValueType
    @property
    def script_config(self) -> global___BTCScriptConfig: ...
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(self,
        *,
        coin: global___BTCCoin.ValueType = ...,
        script_config: typing.Optional[global___BTCScriptConfig] = ...,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["script_config",b"script_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coin",b"coin","keypath",b"keypath","script_config",b"script_config"]) -> None: ...
global___BTCScriptConfigRegistration = BTCScriptConfigRegistration

class BTCSuccess(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___BTCSuccess = BTCSuccess

class BTCIsScriptConfigRegisteredRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REGISTRATION_FIELD_NUMBER: builtins.int
    @property
    def registration(self) -> global___BTCScriptConfigRegistration: ...
    def __init__(self,
        *,
        registration: typing.Optional[global___BTCScriptConfigRegistration] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["registration",b"registration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["registration",b"registration"]) -> None: ...
global___BTCIsScriptConfigRegisteredRequest = BTCIsScriptConfigRegisteredRequest

class BTCIsScriptConfigRegisteredResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    IS_REGISTERED_FIELD_NUMBER: builtins.int
    is_registered: builtins.bool
    def __init__(self,
        *,
        is_registered: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["is_registered",b"is_registered"]) -> None: ...
global___BTCIsScriptConfigRegisteredResponse = BTCIsScriptConfigRegisteredResponse

class BTCRegisterScriptConfigRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _XPubType:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _XPubTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCRegisterScriptConfigRequest._XPubType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        AUTO_ELECTRUM: BTCRegisterScriptConfigRequest._XPubType.ValueType  # 0
        """Automatically choose to match Electrum's xpub format (e.g. Zpub/Vpub for p2wsh multisig mainnet/testnet)."""

        AUTO_XPUB_TPUB: BTCRegisterScriptConfigRequest._XPubType.ValueType  # 1
        """Always xpub for mainnets, tpub for testnets."""

    class XPubType(_XPubType, metaclass=_XPubTypeEnumTypeWrapper):
        pass

    AUTO_ELECTRUM: BTCRegisterScriptConfigRequest.XPubType.ValueType  # 0
    """Automatically choose to match Electrum's xpub format (e.g. Zpub/Vpub for p2wsh multisig mainnet/testnet)."""

    AUTO_XPUB_TPUB: BTCRegisterScriptConfigRequest.XPubType.ValueType  # 1
    """Always xpub for mainnets, tpub for testnets."""


    REGISTRATION_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    XPUB_TYPE_FIELD_NUMBER: builtins.int
    @property
    def registration(self) -> global___BTCScriptConfigRegistration: ...
    name: typing.Text
    """If empty, the name is entered on the device instead."""

    xpub_type: global___BTCRegisterScriptConfigRequest.XPubType.ValueType
    def __init__(self,
        *,
        registration: typing.Optional[global___BTCScriptConfigRegistration] = ...,
        name: typing.Text = ...,
        xpub_type: global___BTCRegisterScriptConfigRequest.XPubType.ValueType = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["registration",b"registration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["name",b"name","registration",b"registration","xpub_type",b"xpub_type"]) -> None: ...
global___BTCRegisterScriptConfigRequest = BTCRegisterScriptConfigRequest

class BTCPrevTxInitRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VERSION_FIELD_NUMBER: builtins.int
    NUM_INPUTS_FIELD_NUMBER: builtins.int
    NUM_OUTPUTS_FIELD_NUMBER: builtins.int
    LOCKTIME_FIELD_NUMBER: builtins.int
    version: builtins.int
    num_inputs: builtins.int
    num_outputs: builtins.int
    locktime: builtins.int
    def __init__(self,
        *,
        version: builtins.int = ...,
        num_inputs: builtins.int = ...,
        num_outputs: builtins.int = ...,
        locktime: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["locktime",b"locktime","num_inputs",b"num_inputs","num_outputs",b"num_outputs","version",b"version"]) -> None: ...
global___BTCPrevTxInitRequest = BTCPrevTxInitRequest

class BTCPrevTxInputRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREV_OUT_HASH_FIELD_NUMBER: builtins.int
    PREV_OUT_INDEX_FIELD_NUMBER: builtins.int
    SIGNATURE_SCRIPT_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    prev_out_hash: builtins.bytes
    prev_out_index: builtins.int
    signature_script: builtins.bytes
    sequence: builtins.int
    def __init__(self,
        *,
        prev_out_hash: builtins.bytes = ...,
        prev_out_index: builtins.int = ...,
        signature_script: builtins.bytes = ...,
        sequence: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prev_out_hash",b"prev_out_hash","prev_out_index",b"prev_out_index","sequence",b"sequence","signature_script",b"signature_script"]) -> None: ...
global___BTCPrevTxInputRequest = BTCPrevTxInputRequest

class BTCPrevTxOutputRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALUE_FIELD_NUMBER: builtins.int
    PUBKEY_SCRIPT_FIELD_NUMBER: builtins.int
    value: builtins.int
    pubkey_script: builtins.bytes
    def __init__(self,
        *,
        value: builtins.int = ...,
        pubkey_script: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pubkey_script",b"pubkey_script","value",b"value"]) -> None: ...
global___BTCPrevTxOutputRequest = BTCPrevTxOutputRequest

class BTCSignMessageRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COIN_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    MSG_FIELD_NUMBER: builtins.int
    HOST_NONCE_COMMITMENT_FIELD_NUMBER: builtins.int
    coin: global___BTCCoin.ValueType
    @property
    def script_config(self) -> global___BTCScriptConfigWithKeypath: ...
    msg: builtins.bytes
    @property
    def host_nonce_commitment(self) -> antiklepto_pb2.AntiKleptoHostNonceCommitment: ...
    def __init__(self,
        *,
        coin: global___BTCCoin.ValueType = ...,
        script_config: typing.Optional[global___BTCScriptConfigWithKeypath] = ...,
        msg: builtins.bytes = ...,
        host_nonce_commitment: typing.Optional[antiklepto_pb2.AntiKleptoHostNonceCommitment] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["host_nonce_commitment",b"host_nonce_commitment","script_config",b"script_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coin",b"coin","host_nonce_commitment",b"host_nonce_commitment","msg",b"msg","script_config",b"script_config"]) -> None: ...
global___BTCSignMessageRequest = BTCSignMessageRequest

class BTCSignMessageResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SIGNATURE_FIELD_NUMBER: builtins.int
    signature: builtins.bytes
    """65 bytes (32 bytes big endian R, 32 bytes big endian S, 1 recid)."""

    def __init__(self,
        *,
        signature: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["signature",b"signature"]) -> None: ...
global___BTCSignMessageResponse = BTCSignMessageResponse

class BTCRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    IS_SCRIPT_CONFIG_REGISTERED_FIELD_NUMBER: builtins.int
    REGISTER_SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    PREVTX_INIT_FIELD_NUMBER: builtins.int
    PREVTX_INPUT_FIELD_NUMBER: builtins.int
    PREVTX_OUTPUT_FIELD_NUMBER: builtins.int
    SIGN_MESSAGE_FIELD_NUMBER: builtins.int
    ANTIKLEPTO_SIGNATURE_FIELD_NUMBER: builtins.int
    @property
    def is_script_config_registered(self) -> global___BTCIsScriptConfigRegisteredRequest: ...
    @property
    def register_script_config(self) -> global___BTCRegisterScriptConfigRequest: ...
    @property
    def prevtx_init(self) -> global___BTCPrevTxInitRequest: ...
    @property
    def prevtx_input(self) -> global___BTCPrevTxInputRequest: ...
    @property
    def prevtx_output(self) -> global___BTCPrevTxOutputRequest: ...
    @property
    def sign_message(self) -> global___BTCSignMessageRequest: ...
    @property
    def antiklepto_signature(self) -> antiklepto_pb2.AntiKleptoSignatureRequest: ...
    def __init__(self,
        *,
        is_script_config_registered: typing.Optional[global___BTCIsScriptConfigRegisteredRequest] = ...,
        register_script_config: typing.Optional[global___BTCRegisterScriptConfigRequest] = ...,
        prevtx_init: typing.Optional[global___BTCPrevTxInitRequest] = ...,
        prevtx_input: typing.Optional[global___BTCPrevTxInputRequest] = ...,
        prevtx_output: typing.Optional[global___BTCPrevTxOutputRequest] = ...,
        sign_message: typing.Optional[global___BTCSignMessageRequest] = ...,
        antiklepto_signature: typing.Optional[antiklepto_pb2.AntiKleptoSignatureRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["antiklepto_signature",b"antiklepto_signature","is_script_config_registered",b"is_script_config_registered","prevtx_init",b"prevtx_init","prevtx_input",b"prevtx_input","prevtx_output",b"prevtx_output","register_script_config",b"register_script_config","request",b"request","sign_message",b"sign_message"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["antiklepto_signature",b"antiklepto_signature","is_script_config_registered",b"is_script_config_registered","prevtx_init",b"prevtx_init","prevtx_input",b"prevtx_input","prevtx_output",b"prevtx_output","register_script_config",b"register_script_config","request",b"request","sign_message",b"sign_message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["request",b"request"]) -> typing.Optional[typing_extensions.Literal["is_script_config_registered","register_script_config","prevtx_init","prevtx_input","prevtx_output","sign_message","antiklepto_signature"]]: ...
global___BTCRequest = BTCRequest

class BTCResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SUCCESS_FIELD_NUMBER: builtins.int
    IS_SCRIPT_CONFIG_REGISTERED_FIELD_NUMBER: builtins.int
    SIGN_NEXT_FIELD_NUMBER: builtins.int
    SIGN_MESSAGE_FIELD_NUMBER: builtins.int
    ANTIKLEPTO_SIGNER_COMMITMENT_FIELD_NUMBER: builtins.int
    @property
    def success(self) -> global___BTCSuccess: ...
    @property
    def is_script_config_registered(self) -> global___BTCIsScriptConfigRegisteredResponse: ...
    @property
    def sign_next(self) -> global___BTCSignNextResponse: ...
    @property
    def sign_message(self) -> global___BTCSignMessageResponse: ...
    @property
    def antiklepto_signer_commitment(self) -> antiklepto_pb2.AntiKleptoSignerCommitment: ...
    def __init__(self,
        *,
        success: typing.Optional[global___BTCSuccess] = ...,
        is_script_config_registered: typing.Optional[global___BTCIsScriptConfigRegisteredResponse] = ...,
        sign_next: typing.Optional[global___BTCSignNextResponse] = ...,
        sign_message: typing.Optional[global___BTCSignMessageResponse] = ...,
        antiklepto_signer_commitment: typing.Optional[antiklepto_pb2.AntiKleptoSignerCommitment] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["antiklepto_signer_commitment",b"antiklepto_signer_commitment","is_script_config_registered",b"is_script_config_registered","response",b"response","sign_message",b"sign_message","sign_next",b"sign_next","success",b"success"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["antiklepto_signer_commitment",b"antiklepto_signer_commitment","is_script_config_registered",b"is_script_config_registered","response",b"response","sign_message",b"sign_message","sign_next",b"sign_next","success",b"success"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["response",b"response"]) -> typing.Optional[typing_extensions.Literal["success","is_script_config_registered","sign_next","sign_message","antiklepto_signer_commitment"]]: ...
global___BTCResponse = BTCResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
from . import common_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _CardanoNetwork:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _CardanoNetworkEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CardanoNetwork.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    CardanoMainnet: _CardanoNetwork.ValueType  # 0
    CardanoTestnet: _CardanoNetwork.ValueType  # 1
class CardanoNetwork(_CardanoNetwork, metaclass=_CardanoNetworkEnumTypeWrapper):
    pass

CardanoMainnet: CardanoNetwork.ValueType  # 0
CardanoTestnet: CardanoNetwork.ValueType  # 1
global___CardanoNetwork = CardanoNetwork


class CardanoXpubsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEYPATHS_FIELD_NUMBER: builtins.int
    @property
    def keypaths(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common_pb2.Keypath]: ...
    def __init__(self,
        *,
        keypaths: typing.Optional[typing.Iterable[common_pb2.Keypath]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["keypaths",b"keypaths"]) -> None: ...
global___CardanoXpubsRequest = CardanoXpubsRequest

class CardanoXpubsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    XPUBS_FIELD_NUMBER: builtins.int
    @property
    def xpubs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(self,
        *,
        xpubs: typing.Optional[typing.Iterable[builtins.bytes]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["xpubs",b"xpubs"]) -> None: ...
global___CardanoXpubsResponse = CardanoXpubsResponse

class CardanoScriptConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class PkhSkh(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEYPATH_PAYMENT_FIELD_NUMBER: builtins.int
        KEYPATH_STAKE_FIELD_NUMBER: builtins.int
        @property
        def keypath_payment(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        @property
        def keypath_stake(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(self,
            *,
            keypath_payment: typing.Optional[typing.Iterable[builtins.int]] = ...,
            keypath_stake: typing.Optional[typing.Iterable[builtins.int]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["keypath_payment",b"keypath_payment","keypath_stake",b"keypath_stake"]) -> None: ...

    PKH_SKH_FIELD_NUMBER: builtins.int
    @property
    def pkh_skh(self) -> global___CardanoScriptConfig.PkhSkh:
        """Shelley PaymentKeyHash & StakeKeyHash"""
        pass
    def __init__(self,
        *,
        pkh_skh: typing.Optional[global___CardanoScriptConfig.PkhSkh] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["config",b"config","pkh_skh",b"pkh_skh"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["config",b"config","pkh_skh",b"pkh_skh"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["config",b"config"]) -> typing.Optional[typing_extensions.Literal["pkh_skh"]]: ...
global___CardanoScriptConfig = CardanoScriptConfig

class CardanoAddressRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NETWORK_FIELD_NUMBER: builtins.int
    DISPLAY_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    network: global___CardanoNetwork.ValueType
    display: builtins.bool
    @property
    def script_config(self) -> global___CardanoScriptConfig: ...
    def __init__(self,
        *,
        network: global___CardanoNetwork.ValueType = ...,
        display: builtins.bool = ...,
        script_config: typing.Optional[global___CardanoScriptConfig] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["script_config",b"script_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["display",b"display","network",b"network","script_config",b"script_config"]) -> None: ...
global___CardanoAddressRequest = CardanoAddressRequest

class CardanoSignTransactionRequest(google.protobuf.message.Message):
    """Max allowed transaction size is 16384 bytes according to
    https://github.com/cardano-foundation/CIPs/blob/master/CIP-0009/CIP-0009.md. Unlike with BTC, we
    can fit the whole request in RAM and don't need to stream.

    See also: https://github.com/input-output-hk/cardano-ledger-specs/blob/d0aa86ded0b973b09b629e5aa62aa1e71364d088/eras/alonzo/test-suite/cddl-files/alonzo.cddl#L50
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class Input(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEYPATH_FIELD_NUMBER: builtins.int
        PREV_OUT_HASH_FIELD_NUMBER: builtins.int
        PREV_OUT_INDEX_FIELD_NUMBER: builtins.int
        @property
        def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        prev_out_hash: builtins.bytes
        prev_out_index: builtins.int
        def __init__(self,
            *,
            keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
            prev_out_hash: builtins.bytes = ...,
            prev_out_index: builtins.int = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["keypath",b"keypath","prev_out_hash",b"prev_out_hash","prev_out_index",b"prev_out_index"]) -> None: ...

    class AssetGroup(google.protobuf.message.Message):
        """https://github.com/input-output-hk/cardano-ledger/blob/d0aa86ded0b973b09b629e5aa62aa1e71364d088/eras/alonzo/test-suite/cddl-files/alonzo.cddl#L358"""
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        class Token(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor
            ASSET_NAME_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            asset_name: builtins.bytes
            value: builtins.int
            """Number of tokens transacted of this asset."""

            def __init__(self,
                *,
                asset_name: builtins.bytes = ...,
                value: builtins.int = ...,
                ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["asset_name",b"asset_name","value",b"value"]) -> None: ...

        POLICY_ID_FIELD_NUMBER: builtins.int
        TOKENS_FIELD_NUMBER: builtins.int
        policy_id: builtins.bytes
        @property
        def tokens(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CardanoSignTransactionRequest.AssetGroup.Token]: ...
        def __init__(self,
            *,
            policy_id: builtins.bytes = ...,
            tokens: typing.Optional[typing.Iterable[global___CardanoSignTransactionRequest.AssetGroup.Token]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["policy_id",b"policy_id","tokens",b"tokens"]) -> None: ...

    class Output(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        ENCODED_ADDRESS_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
        ASSET_GROUPS_FIELD_NUMBER: builtins.int
        encoded_address: typing.Text
        value: builtins.int
        @property
        def script_config(self) -> global___CardanoScriptConfig:
            """Optional. If provided, this is validated as a change output."""
            pass
        @property
        def asset_groups(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CardanoSignTransactionRequest.AssetGroup]: ...
        def __init__(self,
            *,
            encoded_address: typing.Text = ...,
            value: builtins.int = ...,
            script_config: typing.Optional[global___CardanoScriptConfig] = ...,
            asset_groups: typing.Optional[typing.Iterable[global___CardanoSignTransactionRequest.AssetGroup]] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["script_config",b"script_config"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["asset_groups",b"asset_groups","encoded_address",b"encoded_address","script_config",b"script_config","value",b"value"]) -> None: ...

    class Certificate(google.protobuf.message.Message):
        """See https://github.com/input-output-hk/cardano-ledger-specs/blob/d0aa86ded0b973b09b629e5aa62aa1e71364d088/eras/alonzo/test-suite/cddl-files/alonzo.cddl#L150"""
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        class StakeDelegation(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor
            KEYPATH_FIELD_NUMBER: builtins.int
            POOL_KEYHASH_FIELD_NUMBER: builtins.int
            @property
            def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
            pool_keyhash: builtins.bytes
            def __init__(self,
                *,
                keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
                pool_keyhash: builtins.bytes = ...,
                ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["keypath",b"keypath","pool_keyhash",b"pool_keyhash"]) -> None: ...

        STAKE_REGISTRATION_FIELD_NUMBER: builtins.int
        STAKE_DEREGISTRATION_FIELD_NUMBER: builtins.int
        STAKE_DELEGATION_FIELD_NUMBER: builtins.int
        @property
        def stake_registration(self) -> common_pb2.Keypath: ...
        @property
        def stake_deregistration(self) -> common_pb2.Keypath: ...
        @property
        def stake_delegation(self) -> global___CardanoSignTransactionRequest.Certificate.StakeDelegation: ...
        def __init__(self,
            *,
            stake_registration: typing.Optional[common_pb2.Keypath] = ...,
            stake_deregistration: typing.Optional[common_pb2.Keypath] = ...,
            stake_delegation: typing.Optional[global___CardanoSignTransactionRequest.Certificate.StakeDelegation] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["cert",b"cert","stake_delegation",b"stake_delegation","stake_deregistration",b"stake_deregistration","stake_registration",b"stake_registration"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["cert",b"cert","stake_delegation",b"stake_delegation","stake_deregistration",b"stake_deregistration","stake_registration",b"stake_registration"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal["cert",b"cert"]) -> typing.Optional[typing_extensions.Literal["stake_registration","stake_deregistration","stake_delegation"]]: ...

    class Withdrawal(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEYPATH_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        @property
        def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        value: builtins.int
        def __init__(self,
            *,
            keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
            value: builtins.int = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["keypath",b"keypath","value",b"value"]) -> None: ...

    NETWORK_FIELD_NUMBER: builtins.int
    INPUTS_FIELD_NUMBER: builtins.int
    OUTPUTS_FIELD_NUMBER: builtins.int
    FEE_FIELD_NUMBER: builtins.int
    TTL_FIELD_NUMBER: builtins.int
    CERTIFICATES_FIELD_NUMBER: builtins.int
    WITHDRAWALS_FIELD_NUMBER: builtins.int
    VALIDITY_INTERVAL_START_FIELD_NUMBER: builtins.int
    ALLOW_ZERO_TTL_FIELD_NUMBER: builtins.int
    network: global___CardanoNetwork.ValueType
    @property
    def inputs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CardanoSignTransactionRequest.Input]: ...
    @property
    def outputs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CardanoSignTransactionRequest.Output]: ...
    fee: builtins.int
    ttl: builtins.int
    @property
    def certificates(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CardanoSignTransactionRequest.Certificate]: ...
    @property
    def withdrawals(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CardanoSignTransactionRequest.Withdrawal]: ...
    validity_interval_start: builtins.int
    allow_zero_ttl: builtins.bool
    """include ttl even if it is zero"""

    def __init__(self,
        *,
        network: global___CardanoNetwork.ValueType = ...,
        inputs: typing.Optional[typing.Iterable[global___CardanoSignTransactionRequest.Input]] = ...,
        outputs: typing.Optional[typing.Iterable[global___CardanoSignTransactionRequest.Output]] = ...,
        fee: builtins.int = ...,
        ttl: builtins.int = ...,
        certificates: typing.Optional[typing.Iterable[global___CardanoSignTransactionRequest.Certificate]] = ...,
        withdrawals: typing.Optional[typing.Iterable[global___CardanoSignTransactionRequest.Withdrawal]] = ...,
        validity_interval_start: builtins.int = ...,
        allow_zero_ttl: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_zero_ttl",b"allow_zero_ttl","certificates",b"certificates","fee",b"fee","inputs",b"inputs","network",b"network","outputs",b"outputs","ttl",b"ttl","validity_interval_start",b"validity_interval_start","withdrawals",b"withdrawals"]) -> None: ...
global___CardanoSignTransactionRequest = CardanoSignTransactionRequest

class CardanoSignTransactionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class ShelleyWitness(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        PUBLIC_KEY_FIELD_NUMBER: builtins.int
        SIGNATURE_FIELD_NUMBER: builtins.int
        public_key: builtins.bytes
        signature: builtins.bytes
        def __init__(self,
            *,
            public_key: builtins.bytes = ...,
            signature: builtins.bytes = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["public_key",b"public_key","signature",b"signature"]) -> None: ...

    SHELLEY_WITNESSES_FIELD_NUMBER: builtins.int
    @property
    def shelley_witnesses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CardanoSignTransactionResponse.ShelleyWitness]: ...
    def __init__(self,
        *,
        shelley_witnesses: typing.Optional[typing.Iterable[global___CardanoSignTransactionResponse.ShelleyWitness]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["shelley_witnesses",b"shelley_witnesses"]) -> None: ...
global___CardanoSignTransactionResponse = CardanoSignTransactionResponse

class CardanoRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    XPUBS_FIELD_NUMBER: builtins.int
    ADDRESS_FIELD_NUMBER: builtins.int
    SIGN_TRANSACTION_FIELD_NUMBER: builtins.int
    @property
    def xpubs(self) -> global___CardanoXpubsRequest: ...
    @property
    def address(self) -> global___CardanoAddressRequest: ...
    @property
    def sign_transaction(self) -> global___CardanoSignTransactionRequest: ...
    def __init__(self,
        *,
        xpubs: typing.Optional[global___CardanoXpubsRequest] = ...,
        address: typing.Optional[global___CardanoAddressRequest] = ...,
        sign_transaction: typing.Optional[global___CardanoSignTransactionRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["address",b"address","request",b"request","sign_transaction",b"sign_transaction","xpubs",b"xpubs"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["address",b"address","request",b"request","sign_transaction",b"sign_transaction","xpubs",b"xpubs"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["request",b"request"]) -> typing.Optional[typing_extensions.Literal["xpubs","address","sign_transaction"]]: ...
global___CardanoRequest = CardanoRequest

class CardanoResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    XPUBS_FIELD_NUMBER: builtins.int
    PUB_FIELD_NUMBER: builtins.int
    SIGN_TRANSACTION_FIELD_NUMBER: builtins.int
    @property
    def xpubs(self) -> global___CardanoXpubsResponse: ...
    @property
    def pub(self) -> common_pb2.PubResponse: ...
    @property
    def sign_transaction(self) -> global___CardanoSignTransactionResponse: ...
    def __init__(self,
        *,
        xpubs: typing.Optional[global___CardanoXpubsResponse] = ...,
        pub: typing.Optional[common_pb2.PubResponse] = ...,
        sign_transaction: typing.Optional[global___CardanoSignTransactionResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pub",b"pub","response",b"response","sign_transaction",b"sign_transaction","xpubs",b"xpubs"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pub",b"pub","response",b"response","sign_transaction",b"sign_transaction","xpubs",b"xpubs"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["response",b"response"]) -> typing.Optional[typing_extensions.Literal["xpubs","pub","sign_transaction"]]: ...
global___CardanoResponse = CardanoResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ElectrumEncryptionKeyRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEYPATH_FIELD_NUMBER: builtins.int
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(self,
        *,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["keypath",b"keypath"]) -> None: ...
global___ElectrumEncryptionKeyRequest = ElectrumEncryptionKeyRequest

class ElectrumEncryptionKeyResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    key: typing.Text
    def __init__(self,
        *,
        key: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key",b"key"]) -> None: ...
global___ElectrumEncryptionKeyResponse = ElectrumEncryptionKeyResponse



"""ledgercomm.comm module."""

from abc import ABCMeta, abstractmethod
from typing import Tuple


class Comm(metaclass=ABCMeta):
    """Abstract class for communication interface."""

    @abstractmethod
    def open(self) -> None:
        """Just open the interface."""
        raise NotImplementedError

    @abstractmethod
    def send(self, data: bytes) -> int:
        """Allow to send raw bytes from the interface."""
        raise NotImplementedError

    @abstractmethod
    def recv(self) -> Tuple[int, bytes]:
        """Allow to receive raw bytes from the interface."""
        raise NotImplementedError

    @abstractmethod
    def exchange(self, data: bytes) -> Tuple[int, bytes]:
        """Allow to send and receive raw bytes from the interface."""
        raise NotImplementedError

    @abstractmethod
    def close(self) -> None:
        """Just close the interface."""
        raise NotImplementedError



# Copyright 2019 Shift Cryptosecurity AG
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Implementations"""

import struct
import random
from ..communication import PhysicalLayer, TransportLayer

USB_REPORT_SIZE = 64

ERR_NONE = 0x00
ERR_INVALID_CMD = 0x01
ERR_INVALID_PAR = 0x02
ERR_INVALID_LEN = 0x03
ERR_INVALID_SEQ = 0x04
ERR_MSG_TIMEOUT = 0x05
ERR_CHANNEL_BUSY = 0x06
ERR_LOCK_REQUIRED = 0x0A
ERR_INVALID_CID = 0x0B
ERR_OTHER = 0x7F

PING = 0x80 | 0x01
MSG = 0x80 | 0x03
LOCK = 0x80 | 0x04
INIT = 0x80 | 0x06
WINK = 0x80 | 0x08
SYNC = 0x80 | 0x3C
ERROR = 0x80 | 0x3F

CID_BROADCAST = 0xFFFFFFFF


class U2FHid(TransportLayer):
    """U2F-over-HID transport layer."""

    def __init__(self, device: PhysicalLayer):
        self._device = device

    def generate_cid(self) -> int:
        """Generate a valid CID"""
        # Exclude 0 and u32_max (0xffff_ffff)
        return random.randrange(1, 0xFFFFFFFF)

    # TODO: Create exceptions
    def _throw_error(self, error_code: int) -> None:
        if error_code == ERR_INVALID_CMD:
            raise Exception("Received error: invalid command")
        if error_code == ERR_INVALID_LEN:
            raise Exception("Received error: invalid length")
        if error_code == ERR_INVALID_SEQ:
            raise Exception("Received error: invalid sequence")
        if error_code == ERR_CHANNEL_BUSY:
            raise Exception("Received error: channel busy")
        if error_code == 0x7E:
            raise Exception("Received error: encryption failed")
        if error_code == ERR_OTHER:
            raise Exception("Received unknown error")
        raise Exception("Received error: %d" % error_code)

    def write(self, data: bytes, endpoint: int, cid: int) -> None:
        """
        Send data to the device.

        Args:
            data: Data to send
            endpoint: U2F HID command (endpoint selection)
            cid: U2F HID channel ID
        Throws:
            ValueError: In case any value is out of range
        """
        if endpoint < 0 or endpoint > 0xFF:
            raise ValueError("Channel command (endpoint) is out of range '0 < endpoint <= 0xFF'")
        if cid < 0 or cid > 0xFFFFFFFF:
            raise ValueError("Channel id is out of range '0 < cid <= 0xFFFFFFFF'")
        data = bytearray(data)
        data_len = len(data)
        if data_len > 0xFFFF:
            raise ValueError("Data is too large 'size <= 0xFFFF'")
        seq = 0
        idx = 0
        buf = b""
        # Allow to write an empty packet
        single_empty_write = data_len == 0
        while idx < data_len or single_empty_write:
            if idx == 0:
                # INIT frame
                buf = data[idx : idx + min(data_len, USB_REPORT_SIZE - 7)]
                self._device.write(
                    b"\0"
                    + struct.pack(">IBH", cid, endpoint, data_len)
                    + buf
                    + b"\xEE" * (USB_REPORT_SIZE - 7 - len(buf))
                )
            else:
                # CONT frame
                buf = data[idx : idx + min(data_len, USB_REPORT_SIZE - 5)]
                self._device.write(
                    b"\0"
                    + struct.pack(">IB", cid, seq)
                    + buf
                    + b"\xEE" * (USB_REPORT_SIZE - 5 - len(buf))
                )
                seq += 1
            idx += len(buf)
            single_empty_write = False

    def read(self, endpoint: int, cid: int) -> bytes:
        """
        Receive data from the device.

        Args:
            endpoint: The expected returned U2F HID command
            cid: The expected returned U2F HID channel ID
        Returns:
            The read message combined from the u2fhid packets
        Throws:
            ValueError: In case any value is out of range
            Exception: In case of USB communication issues
        """
        if endpoint < 0 or endpoint > 0xFF:
            raise ValueError("Endpoint/U2F command is out of range '0 < endpoint <= 0xFF'")
        if cid < 0 or cid > 0xFFFFFFFF:
            raise ValueError("Channel id is out of range '0 < cid <= 0xFFFFFFFF'")
        timeout_ms = 5000000
        buf = self._device.read(USB_REPORT_SIZE, timeout_ms)
        if len(buf) >= 3:
            reply_cid = ((buf[0] * 256 + buf[1]) * 256 + buf[2]) * 256 + buf[3]
            reply_cmd = buf[4]
            data_len = buf[5] * 256 + buf[6]
            data = buf[7:]
            idx = len(buf) - 7
            if reply_cmd == ERROR:
                self._throw_error(data[0])
            while idx < data_len:
                # CONT response
                buf = self._device.read(USB_REPORT_SIZE, timeout_ms)
                if len(buf) < 3:
                    raise Exception("Did not receive a continuation frame after 5000 seconds.")
                data += buf[5:]
                idx += len(buf) - 5
            if reply_cid != cid:
                raise Exception(f"- USB channel ID mismatch {reply_cid:x} != {cid:x}")
            if reply_cmd != endpoint:
                raise Exception(f"- USB command mismatch {reply_cmd:x} != {endpoint:x}")
            return bytes(data[:data_len])
        raise Exception("Did not read anything after 5000 seconds.")

    def close(self) -> None:
        self._device.close()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ShowMnemonicRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___ShowMnemonicRequest = ShowMnemonicRequest

class RestoreFromMnemonicRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMEZONE_OFFSET_FIELD_NUMBER: builtins.int
    timestamp: builtins.int
    timezone_offset: builtins.int
    def __init__(self,
        *,
        timestamp: builtins.int = ...,
        timezone_offset: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["timestamp",b"timestamp","timezone_offset",b"timezone_offset"]) -> None: ...
global___RestoreFromMnemonicRequest = RestoreFromMnemonicRequest

class SetMnemonicPassphraseEnabledRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ENABLED_FIELD_NUMBER: builtins.int
    enabled: builtins.bool
    def __init__(self,
        *,
        enabled: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["enabled",b"enabled"]) -> None: ...
global___SetMnemonicPassphraseEnabledRequest = SetMnemonicPassphraseEnabledRequest



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: system.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0csystem.proto\x12\x14shiftcrypto.bitbox02\"s\n\rRebootRequest\x12<\n\x07purpose\x18\x01 \x01(\x0e\x32+.shiftcrypto.bitbox02.RebootRequest.Purpose\"$\n\x07Purpose\x12\x0b\n\x07UPGRADE\x10\x00\x12\x0c\n\x08SETTINGS\x10\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'system_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _REBOOTREQUEST._serialized_start=38
  _REBOOTREQUEST._serialized_end=153
  _REBOOTREQUEST_PURPOSE._serialized_start=117
  _REBOOTREQUEST_PURPOSE._serialized_end=153
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
from . import antiklepto_pb2
import builtins
from . import common_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _ETHCoin:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _ETHCoinEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ETHCoin.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ETH: _ETHCoin.ValueType  # 0
    RopstenETH: _ETHCoin.ValueType  # 1
    """Removed in v9.14.0 - deprecated"""

    RinkebyETH: _ETHCoin.ValueType  # 2
    """Removed in v9.14.0 - deprecated"""

class ETHCoin(_ETHCoin, metaclass=_ETHCoinEnumTypeWrapper):
    """Kept for backwards compatibility. Use chain_id instead, introduced in v9.10.0."""
    pass

ETH: ETHCoin.ValueType  # 0
RopstenETH: ETHCoin.ValueType  # 1
"""Removed in v9.14.0 - deprecated"""

RinkebyETH: ETHCoin.ValueType  # 2
"""Removed in v9.14.0 - deprecated"""

global___ETHCoin = ETHCoin


class ETHPubRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _OutputType:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _OutputTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ETHPubRequest._OutputType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        ADDRESS: ETHPubRequest._OutputType.ValueType  # 0
        XPUB: ETHPubRequest._OutputType.ValueType  # 1
    class OutputType(_OutputType, metaclass=_OutputTypeEnumTypeWrapper):
        pass

    ADDRESS: ETHPubRequest.OutputType.ValueType  # 0
    XPUB: ETHPubRequest.OutputType.ValueType  # 1

    KEYPATH_FIELD_NUMBER: builtins.int
    COIN_FIELD_NUMBER: builtins.int
    OUTPUT_TYPE_FIELD_NUMBER: builtins.int
    DISPLAY_FIELD_NUMBER: builtins.int
    CONTRACT_ADDRESS_FIELD_NUMBER: builtins.int
    CHAIN_ID_FIELD_NUMBER: builtins.int
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    coin: global___ETHCoin.ValueType
    """Deprecated: use chain_id instead."""

    output_type: global___ETHPubRequest.OutputType.ValueType
    display: builtins.bool
    contract_address: builtins.bytes
    chain_id: builtins.int
    """If non-zero, `coin` is ignored and `chain_id` is used to identify the network."""

    def __init__(self,
        *,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        coin: global___ETHCoin.ValueType = ...,
        output_type: global___ETHPubRequest.OutputType.ValueType = ...,
        display: builtins.bool = ...,
        contract_address: builtins.bytes = ...,
        chain_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chain_id",b"chain_id","coin",b"coin","contract_address",b"contract_address","display",b"display","keypath",b"keypath","output_type",b"output_type"]) -> None: ...
global___ETHPubRequest = ETHPubRequest

class ETHSignRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COIN_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    NONCE_FIELD_NUMBER: builtins.int
    GAS_PRICE_FIELD_NUMBER: builtins.int
    GAS_LIMIT_FIELD_NUMBER: builtins.int
    RECIPIENT_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    HOST_NONCE_COMMITMENT_FIELD_NUMBER: builtins.int
    CHAIN_ID_FIELD_NUMBER: builtins.int
    coin: global___ETHCoin.ValueType
    """Deprecated: use chain_id instead."""

    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    nonce: builtins.bytes
    """smallest big endian serialization, max. 16 bytes"""

    gas_price: builtins.bytes
    """smallest big endian serialization, max. 16 bytes"""

    gas_limit: builtins.bytes
    """smallest big endian serialization, max. 16 bytes"""

    recipient: builtins.bytes
    """20 byte recipient"""

    value: builtins.bytes
    """smallest big endian serialization, max. 32 bytes"""

    data: builtins.bytes
    @property
    def host_nonce_commitment(self) -> antiklepto_pb2.AntiKleptoHostNonceCommitment: ...
    chain_id: builtins.int
    """If non-zero, `coin` is ignored and `chain_id` is used to identify the network."""

    def __init__(self,
        *,
        coin: global___ETHCoin.ValueType = ...,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        nonce: builtins.bytes = ...,
        gas_price: builtins.bytes = ...,
        gas_limit: builtins.bytes = ...,
        recipient: builtins.bytes = ...,
        value: builtins.bytes = ...,
        data: builtins.bytes = ...,
        host_nonce_commitment: typing.Optional[antiklepto_pb2.AntiKleptoHostNonceCommitment] = ...,
        chain_id: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["host_nonce_commitment",b"host_nonce_commitment"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chain_id",b"chain_id","coin",b"coin","data",b"data","gas_limit",b"gas_limit","gas_price",b"gas_price","host_nonce_commitment",b"host_nonce_commitment","keypath",b"keypath","nonce",b"nonce","recipient",b"recipient","value",b"value"]) -> None: ...
global___ETHSignRequest = ETHSignRequest

class ETHSignMessageRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COIN_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    MSG_FIELD_NUMBER: builtins.int
    HOST_NONCE_COMMITMENT_FIELD_NUMBER: builtins.int
    CHAIN_ID_FIELD_NUMBER: builtins.int
    coin: global___ETHCoin.ValueType
    """Deprecated: use chain_id instead."""

    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    msg: builtins.bytes
    @property
    def host_nonce_commitment(self) -> antiklepto_pb2.AntiKleptoHostNonceCommitment: ...
    chain_id: builtins.int
    """If non-zero, `coin` is ignored and `chain_id` is used to identify the network."""

    def __init__(self,
        *,
        coin: global___ETHCoin.ValueType = ...,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        msg: builtins.bytes = ...,
        host_nonce_commitment: typing.Optional[antiklepto_pb2.AntiKleptoHostNonceCommitment] = ...,
        chain_id: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["host_nonce_commitment",b"host_nonce_commitment"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chain_id",b"chain_id","coin",b"coin","host_nonce_commitment",b"host_nonce_commitment","keypath",b"keypath","msg",b"msg"]) -> None: ...
global___ETHSignMessageRequest = ETHSignMessageRequest

class ETHSignResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SIGNATURE_FIELD_NUMBER: builtins.int
    signature: builtins.bytes
    """65 bytes, last byte is the recid"""

    def __init__(self,
        *,
        signature: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["signature",b"signature"]) -> None: ...
global___ETHSignResponse = ETHSignResponse

class ETHSignTypedMessageRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _DataType:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _DataTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ETHSignTypedMessageRequest._DataType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: ETHSignTypedMessageRequest._DataType.ValueType  # 0
        BYTES: ETHSignTypedMessageRequest._DataType.ValueType  # 1
        UINT: ETHSignTypedMessageRequest._DataType.ValueType  # 2
        INT: ETHSignTypedMessageRequest._DataType.ValueType  # 3
        BOOL: ETHSignTypedMessageRequest._DataType.ValueType  # 4
        ADDRESS: ETHSignTypedMessageRequest._DataType.ValueType  # 5
        STRING: ETHSignTypedMessageRequest._DataType.ValueType  # 6
        ARRAY: ETHSignTypedMessageRequest._DataType.ValueType  # 7
        STRUCT: ETHSignTypedMessageRequest._DataType.ValueType  # 8
    class DataType(_DataType, metaclass=_DataTypeEnumTypeWrapper):
        pass

    UNKNOWN: ETHSignTypedMessageRequest.DataType.ValueType  # 0
    BYTES: ETHSignTypedMessageRequest.DataType.ValueType  # 1
    UINT: ETHSignTypedMessageRequest.DataType.ValueType  # 2
    INT: ETHSignTypedMessageRequest.DataType.ValueType  # 3
    BOOL: ETHSignTypedMessageRequest.DataType.ValueType  # 4
    ADDRESS: ETHSignTypedMessageRequest.DataType.ValueType  # 5
    STRING: ETHSignTypedMessageRequest.DataType.ValueType  # 6
    ARRAY: ETHSignTypedMessageRequest.DataType.ValueType  # 7
    STRUCT: ETHSignTypedMessageRequest.DataType.ValueType  # 8

    class MemberType(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        TYPE_FIELD_NUMBER: builtins.int
        SIZE_FIELD_NUMBER: builtins.int
        STRUCT_NAME_FIELD_NUMBER: builtins.int
        ARRAY_TYPE_FIELD_NUMBER: builtins.int
        type: global___ETHSignTypedMessageRequest.DataType.ValueType
        size: builtins.int
        struct_name: typing.Text
        """if type==STRUCT, name of struct type."""

        @property
        def array_type(self) -> global___ETHSignTypedMessageRequest.MemberType:
            """if type==ARRAY, type of elements"""
            pass
        def __init__(self,
            *,
            type: global___ETHSignTypedMessageRequest.DataType.ValueType = ...,
            size: builtins.int = ...,
            struct_name: typing.Text = ...,
            array_type: typing.Optional[global___ETHSignTypedMessageRequest.MemberType] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["array_type",b"array_type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["array_type",b"array_type","size",b"size","struct_name",b"struct_name","type",b"type"]) -> None: ...

    class Member(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        NAME_FIELD_NUMBER: builtins.int
        TYPE_FIELD_NUMBER: builtins.int
        name: typing.Text
        @property
        def type(self) -> global___ETHSignTypedMessageRequest.MemberType: ...
        def __init__(self,
            *,
            name: typing.Text = ...,
            type: typing.Optional[global___ETHSignTypedMessageRequest.MemberType] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["type",b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["name",b"name","type",b"type"]) -> None: ...

    class StructType(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        NAME_FIELD_NUMBER: builtins.int
        MEMBERS_FIELD_NUMBER: builtins.int
        name: typing.Text
        @property
        def members(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ETHSignTypedMessageRequest.Member]: ...
        def __init__(self,
            *,
            name: typing.Text = ...,
            members: typing.Optional[typing.Iterable[global___ETHSignTypedMessageRequest.Member]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["members",b"members","name",b"name"]) -> None: ...

    CHAIN_ID_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    TYPES_FIELD_NUMBER: builtins.int
    PRIMARY_TYPE_FIELD_NUMBER: builtins.int
    HOST_NONCE_COMMITMENT_FIELD_NUMBER: builtins.int
    chain_id: builtins.int
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def types(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ETHSignTypedMessageRequest.StructType]: ...
    primary_type: typing.Text
    @property
    def host_nonce_commitment(self) -> antiklepto_pb2.AntiKleptoHostNonceCommitment: ...
    def __init__(self,
        *,
        chain_id: builtins.int = ...,
        keypath: typing.Optional[typing.Iterable[builtins.int]] = ...,
        types: typing.Optional[typing.Iterable[global___ETHSignTypedMessageRequest.StructType]] = ...,
        primary_type: typing.Text = ...,
        host_nonce_commitment: typing.Optional[antiklepto_pb2.AntiKleptoHostNonceCommitment] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["host_nonce_commitment",b"host_nonce_commitment"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chain_id",b"chain_id","host_nonce_commitment",b"host_nonce_commitment","keypath",b"keypath","primary_type",b"primary_type","types",b"types"]) -> None: ...
global___ETHSignTypedMessageRequest = ETHSignTypedMessageRequest

class ETHTypedMessageValueResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _RootObject:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _RootObjectEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ETHTypedMessageValueResponse._RootObject.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: ETHTypedMessageValueResponse._RootObject.ValueType  # 0
        DOMAIN: ETHTypedMessageValueResponse._RootObject.ValueType  # 1
        MESSAGE: ETHTypedMessageValueResponse._RootObject.ValueType  # 2
    class RootObject(_RootObject, metaclass=_RootObjectEnumTypeWrapper):
        pass

    UNKNOWN: ETHTypedMessageValueResponse.RootObject.ValueType  # 0
    DOMAIN: ETHTypedMessageValueResponse.RootObject.ValueType  # 1
    MESSAGE: ETHTypedMessageValueResponse.RootObject.ValueType  # 2

    ROOT_OBJECT_FIELD_NUMBER: builtins.int
    PATH_FIELD_NUMBER: builtins.int
    root_object: global___ETHTypedMessageValueResponse.RootObject.ValueType
    @property
    def path(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(self,
        *,
        root_object: global___ETHTypedMessageValueResponse.RootObject.ValueType = ...,
        path: typing.Optional[typing.Iterable[builtins.int]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["path",b"path","root_object",b"root_object"]) -> None: ...
global___ETHTypedMessageValueResponse = ETHTypedMessageValueResponse

class ETHTypedMessageValueRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.bytes
    def __init__(self,
        *,
        value: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___ETHTypedMessageValueRequest = ETHTypedMessageValueRequest

class ETHRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PUB_FIELD_NUMBER: builtins.int
    SIGN_FIELD_NUMBER: builtins.int
    SIGN_MSG_FIELD_NUMBER: builtins.int
    ANTIKLEPTO_SIGNATURE_FIELD_NUMBER: builtins.int
    SIGN_TYPED_MSG_FIELD_NUMBER: builtins.int
    TYPED_MSG_VALUE_FIELD_NUMBER: builtins.int
    @property
    def pub(self) -> global___ETHPubRequest: ...
    @property
    def sign(self) -> global___ETHSignRequest: ...
    @property
    def sign_msg(self) -> global___ETHSignMessageRequest: ...
    @property
    def antiklepto_signature(self) -> antiklepto_pb2.AntiKleptoSignatureRequest: ...
    @property
    def sign_typed_msg(self) -> global___ETHSignTypedMessageRequest: ...
    @property
    def typed_msg_value(self) -> global___ETHTypedMessageValueRequest: ...
    def __init__(self,
        *,
        pub: typing.Optional[global___ETHPubRequest] = ...,
        sign: typing.Optional[global___ETHSignRequest] = ...,
        sign_msg: typing.Optional[global___ETHSignMessageRequest] = ...,
        antiklepto_signature: typing.Optional[antiklepto_pb2.AntiKleptoSignatureRequest] = ...,
        sign_typed_msg: typing.Optional[global___ETHSignTypedMessageRequest] = ...,
        typed_msg_value: typing.Optional[global___ETHTypedMessageValueRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["antiklepto_signature",b"antiklepto_signature","pub",b"pub","request",b"request","sign",b"sign","sign_msg",b"sign_msg","sign_typed_msg",b"sign_typed_msg","typed_msg_value",b"typed_msg_value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["antiklepto_signature",b"antiklepto_signature","pub",b"pub","request",b"request","sign",b"sign","sign_msg",b"sign_msg","sign_typed_msg",b"sign_typed_msg","typed_msg_value",b"typed_msg_value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["request",b"request"]) -> typing.Optional[typing_extensions.Literal["pub","sign","sign_msg","antiklepto_signature","sign_typed_msg","typed_msg_value"]]: ...
global___ETHRequest = ETHRequest

class ETHResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PUB_FIELD_NUMBER: builtins.int
    SIGN_FIELD_NUMBER: builtins.int
    ANTIKLEPTO_SIGNER_COMMITMENT_FIELD_NUMBER: builtins.int
    TYPED_MSG_VALUE_FIELD_NUMBER: builtins.int
    @property
    def pub(self) -> common_pb2.PubResponse: ...
    @property
    def sign(self) -> global___ETHSignResponse: ...
    @property
    def antiklepto_signer_commitment(self) -> antiklepto_pb2.AntiKleptoSignerCommitment: ...
    @property
    def typed_msg_value(self) -> global___ETHTypedMessageValueResponse: ...
    def __init__(self,
        *,
        pub: typing.Optional[common_pb2.PubResponse] = ...,
        sign: typing.Optional[global___ETHSignResponse] = ...,
        antiklepto_signer_commitment: typing.Optional[antiklepto_pb2.AntiKleptoSignerCommitment] = ...,
        typed_msg_value: typing.Optional[global___ETHTypedMessageValueResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["antiklepto_signer_commitment",b"antiklepto_signer_commitment","pub",b"pub","response",b"response","sign",b"sign","typed_msg_value",b"typed_msg_value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["antiklepto_signer_commitment",b"antiklepto_signer_commitment","pub",b"pub","response",b"response","sign",b"sign","typed_msg_value",b"typed_msg_value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["response",b"response"]) -> typing.Optional[typing_extensions.Literal["pub","sign","antiklepto_signer_commitment","typed_msg_value"]]: ...
global___ETHResponse = ETHResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class RebootRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Purpose:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _PurposeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[RebootRequest._Purpose.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UPGRADE: RebootRequest._Purpose.ValueType  # 0
        SETTINGS: RebootRequest._Purpose.ValueType  # 1
    class Purpose(_Purpose, metaclass=_PurposeEnumTypeWrapper):
        pass

    UPGRADE: RebootRequest.Purpose.ValueType  # 0
    SETTINGS: RebootRequest.Purpose.ValueType  # 1

    PURPOSE_FIELD_NUMBER: builtins.int
    purpose: global___RebootRequest.Purpose.ValueType
    def __init__(self,
        *,
        purpose: global___RebootRequest.Purpose.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["purpose",b"purpose"]) -> None: ...
global___RebootRequest = RebootRequest



# Copyright 2019 Shift Cryptosecurity AG
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Useful functions"""

from typing import Any, Dict, Optional, List, Tuple
from pathlib import Path
import json
import os
import binascii
import base64
import platform

from ..._base58 import decode_check
from noise.backends.default.keypairs import KeyPair25519

from .bitbox02 import common
from .communication import bitbox_api_protocol


def parse_xpub(xpub: str) -> common.XPub:
    """
    Parse an xpub to a protobuf XPub.
    The version is stripped, so the xpub can be any format (xpub, ypub, etc.).
    """

    decoded = decode_check(xpub)
    decoded = decoded[4:]
    depth, decoded = decoded[:1], decoded[1:]
    parent_fp, decoded = decoded[:4], decoded[4:]
    child_num, decoded = decoded[:4], decoded[4:]
    chain_code, decoded = decoded[:32], decoded[32:]
    pubkey, decoded = decoded[:33], decoded[33:]
    assert len(decoded) == 0
    return common.XPub(
        depth=depth,
        parent_fingerprint=parent_fp,
        child_num=int.from_bytes(child_num, "big"),
        chain_code=chain_code,
        public_key=pubkey,
    )


class UserCache:
    """Data structure to hold keys"""

    def __init__(self, raw_cache: Optional[str] = None):
        if raw_cache is None:
            self.app_static_privkey = None
            self.device_static_pubkeys: List[bytes] = []
            return
        (privkey, pubkeys) = UserCache.deserialize(raw_cache)
        self.app_static_privkey = privkey
        self.device_static_pubkeys = pubkeys

    def serialize(self) -> str:
        """Serialize struct to string"""
        pubkeys = [binascii.hexlify(x).decode("utf-8") for x in self.device_static_pubkeys]
        privkey = None
        if self.app_static_privkey is not None:
            privkey = binascii.hexlify(self.app_static_privkey).decode("utf-8")
        return json.dumps({"device_static_pubkeys": pubkeys, "app_static_privkey": privkey})

    @staticmethod
    def deserialize(raw: str) -> Tuple[Optional[bytes], List[bytes]]:
        """Deserialize content from disk to struct"""
        try:
            data = json.loads(raw)
            privkey = None
            if data["app_static_privkey"] is not None:
                privkey = binascii.unhexlify(data["app_static_privkey"])
            pubkeys = [binascii.unhexlify(x) for x in data["device_static_pubkeys"]]
            return (privkey, pubkeys)
        except json.JSONDecodeError:
            return (None, [])
        except KeyError:
            return (None, [])


class NoiseConfigUserCache(bitbox_api_protocol.BitBoxNoiseConfig):
    """
    A noise config that stores the keys in a file in XDG_CACHE_HOME or ~/.cache.
    Currently intended as a developer help only (currently no macOS/Windows support).
    """

    def __init__(self, appid: str) -> None:
        """
        Args:
            appid: A string that uniqely identifies your application. It will be used as the name
            of the cache directory. Directory separators will create subdirectories, e.g.
            "shift/test1".
        """
        self._cache_file_path = NoiseConfigUserCache._find_cache_file(appid)
        super().__init__()

    @staticmethod
    def _find_cache_file(appid: str) -> Path:
        cachedir_env = os.environ.get("XDG_CACHE_HOME", "")
        if cachedir_env == "":
            homedir = os.environ.get("HOME", "")
            if homedir == "":
                raise RuntimeError("Can't find cache dir")
            cachedir = Path(homedir) / ".cache"
        else:
            cachedir = Path(cachedir_env)
        return cachedir / appid / "bitbox02.dat"

    def _read_cache(self) -> UserCache:
        try:
            with self._cache_file_path.open("r") as fileh:
                return UserCache(fileh.read())
        except FileNotFoundError:
            return UserCache()

    def _write_cache(self, data: UserCache) -> None:
        self._cache_file_path.parent.mkdir(parents=True, exist_ok=True)
        with self._cache_file_path.open("w") as fileh:
            fileh.write(data.serialize())

    def contains_device_static_pubkey(self, pubkey: bytes) -> bool:
        data = self._read_cache()
        if pubkey in data.device_static_pubkeys:
            return True
        return False

    def add_device_static_pubkey(self, pubkey: bytes) -> None:
        if not self.contains_device_static_pubkey(pubkey):
            data = self._read_cache()
            data.device_static_pubkeys.append(pubkey)
            self._write_cache(data)

    def get_app_static_privkey(self) -> Optional[bytes]:
        data = self._read_cache()
        return data.app_static_privkey

    def set_app_static_privkey(self, privkey: bytes) -> None:
        data = self._read_cache()
        data.app_static_privkey = privkey
        self._write_cache(data)


class BitBoxAppNoiseConfig(bitbox_api_protocol.BitBoxNoiseConfig):
    """
    Noise config that reads and stores the noise keys in the same location as the BitBoxApp.
    This allows a third party BitBox02 integration to re-use the pairing with the BitBoxApp.
    """

    _DEVICE_NOISE_STATIC_PUBKEYS = "deviceNoiseStaticPubkeys"
    _APP_NOISE_STATIC_KEYPAIR = "appNoiseStaticKeypair"

    def __init__(self) -> None:
        system = platform.system()
        if system == "Linux":
            folder = Path(
                os.environ.get(
                    "XDG_CONFIG_HOME",
                    # fallback
                    os.path.join(os.environ["HOME"], ".config"),
                )
            )
        elif system == "Darwin":
            folder = Path(os.environ["HOME"]) / "Library" / "Application Support"
        elif system == "Windows":
            folder = Path(os.environ["APPDATA"])
        else:
            raise NotImplementedError("Unknown system: {}".format(system))
        self._filename = folder / "bitbox" / "bitbox02" / "bitbox02.json"

    def _read(self) -> Any:
        try:
            with self._filename.open("r") as fileh:
                return json.load(fileh)
        except FileNotFoundError:
            return {}
        except json.JSONDecodeError:
            return {}

    def _write(self, data: Dict[str, object]) -> None:
        self._filename.parent.mkdir(parents=True, exist_ok=True)
        with self._filename.open("w") as fileh:
            json.dump(data, fileh)

    def contains_device_static_pubkey(self, pubkey: bytes) -> bool:
        data = self._read()
        return base64.b64encode(pubkey).decode() in data.get(self._DEVICE_NOISE_STATIC_PUBKEYS, [])

    def add_device_static_pubkey(self, pubkey: bytes) -> None:
        if not self.contains_device_static_pubkey(pubkey):
            data = self._read()
            data.setdefault(self._DEVICE_NOISE_STATIC_PUBKEYS, [])
            data[self._DEVICE_NOISE_STATIC_PUBKEYS].append(base64.b64encode(pubkey).decode())
            self._write(data)

    def get_app_static_privkey(self) -> Optional[bytes]:
        data = self._read()
        if self._APP_NOISE_STATIC_KEYPAIR not in data:
            return None

        return base64.b64decode(data[self._APP_NOISE_STATIC_KEYPAIR]["private"])

    def set_app_static_privkey(self, privkey: bytes) -> None:
        data = self._read()
        pubkey = KeyPair25519.from_private_bytes(privkey).public_bytes
        data[self._APP_NOISE_STATIC_KEYPAIR] = {
            "private": base64.b64encode(privkey).decode(),
            "public": base64.b64encode(pubkey).decode(),
        }
        self._write(data)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: keystore.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0ekeystore.proto\x12\x14shiftcrypto.bitbox02\"/\n\x1c\x45lectrumEncryptionKeyRequest\x12\x0f\n\x07keypath\x18\x01 \x03(\r\",\n\x1d\x45lectrumEncryptionKeyResponse\x12\x0b\n\x03key\x18\x01 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'keystore_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ELECTRUMENCRYPTIONKEYREQUEST._serialized_start=40
  _ELECTRUMENCRYPTIONKEYREQUEST._serialized_end=87
  _ELECTRUMENCRYPTIONKEYRESPONSE._serialized_start=89
  _ELECTRUMENCRYPTIONKEYRESPONSE._serialized_end=133
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class AntiKleptoHostNonceCommitment(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COMMITMENT_FIELD_NUMBER: builtins.int
    commitment: builtins.bytes
    def __init__(self,
        *,
        commitment: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commitment",b"commitment"]) -> None: ...
global___AntiKleptoHostNonceCommitment = AntiKleptoHostNonceCommitment

class AntiKleptoSignerCommitment(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COMMITMENT_FIELD_NUMBER: builtins.int
    commitment: builtins.bytes
    def __init__(self,
        *,
        commitment: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commitment",b"commitment"]) -> None: ...
global___AntiKleptoSignerCommitment = AntiKleptoSignerCommitment

class AntiKleptoSignatureRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HOST_NONCE_FIELD_NUMBER: builtins.int
    host_nonce: builtins.bytes
    def __init__(self,
        *,
        host_nonce: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["host_nonce",b"host_nonce"]) -> None: ...
global___AntiKleptoSignatureRequest = AntiKleptoSignatureRequest
